# Set up landmarker image viewer.
# Places in the tcl top window.  Looks for object named viewer.

if {[info commands rtExMath] != "rtExMath"} {

if {$app_type == "exe"} { 
  source ViewerAppTkImageViewerInteractor.tcl
} else {
  source "${landmarker_home}ViewerAppTkImageViewerInteractor.tcl"
}

# Take window level parameters from viewer
proc InitializeWindowLevelInterface {infilename xs ys} {
   global viewer sliceNumber
   global pixmin pixmax
   global landmarker_home
   global deciFactor
   global app_type
   global threshGUI

   # Get parameters from viewer
   set w [viewer GetColorWindow]
   set l [viewer GetColorLevel]
   set sliceNumber [viewer GetZSlice]
   set zMin [viewer GetWholeZMin]
   set zMax [viewer GetWholeZMax]

# Use header info for initial windox and level.
set pixrange [expr $pixmax - $pixmin]
set pixmid [expr ($pixmax + $pixmin) / 2.0]
set l $pixmid
set w [expr abs ($l / 2.0) ]

puts stdout "w: $w"
puts stdout "l: $l"
puts stdout "zMin: $zMin"
puts stdout "zMax: $zMax"
puts stdout "sliceNumber: $sliceNumber"

set sliceNumber [expr $zMax / 2.0]

# Create the image viewer user interface.

destroy .top
toplevel .top -visual best

set filename "Image Viewer: [file tail $infilename]"
wm title .top $filename  

menu .top.menu -type menubar

menu .top.menu.file -tearoff 1
menu .top.menu.edit -tearoff 1
menu .top.menu.options -tearoff 1
menu .top.menu.open -tearoff 1
menu .top.menu.info -tearoff 1
menu .top.menu.help -tearoff 1

.top.menu add cascade -menu .top.menu.file -label "File"
.top.menu add cascade -menu .top.menu.edit -label "Edit"
.top.menu add cascade -menu .top.menu.options -label "Options"
.top.menu add cascade -menu .top.menu.open -label "Open"
.top.menu add cascade -menu .top.menu.info -label "Info"
.top.menu add cascade -menu .top.menu.help -label "Help"

.top.menu.file add command -label Exit -command CloseImageViewer -foreground red
##.top.menu.file add command -label Exit -command ::vtk::cb_exit -foreground red
##.top.menu.options add check -label "Threshold/Isosurface" -command ThresholdImageGUI -variable threshGUI 
.top.menu.open add command -label "Threshold/Isosurface Menu" -command ThresholdImageGUI
set threshGUI 1 
.top.menu.options add check -label "Reverse Colors" -command SetInverseVideo -variable video


set source_file "OpenVolumeSlicer.tcl"
if {$app_type == "exe"} { 
  set source_file $source_file
  .top.menu.open add command -label "Volume Slicer" -command { source "OpenVolumeSlicer.tcl" }
  set source_file "OpenVolumeRenderWithUserInterface.tcl"
  set comm [catch {"source $source_file"} ]
} else {
  set source_file $landmarker_home$source_file
  .top.menu.open add command -label "Volume Slicer" -command { source "${landmarker_home}OpenVolumeSlicer.tcl" }
  set source_file "OpenVolumeRenderWithUserInterface.tcl"
  set comm [catch {"source $landmarker_home$source_file"} ]
}
##.top.menu.open add command -label "Volume Render" -command $comm 

##.top.menu.options add check -label "Window/Level Adjust" -command WindowLevelGUI -variable wlGUI
.top.menu.info add command -label "Image Dimensions" -command DisplayImageDimensions
.top.menu.info add command -label "Image Intensities" -command DisplayImageIntensities

.top configure -menu .top.menu

# The rendering widget
frame .top.f1
vtkTkImageViewerWidget .top.f1.v1 -width $xs -height $ys -iv viewer
pack .top.f1.v1 -padx 3 -pady 3 -side left -fill both -expand t
pack .top.f1 -fill both -expand t

   frame .top.slice
   label .top.slice.label -text " Slice # "
   scale .top.slice.scale -from $zMin -to $zMax -orient horizontal \
     -command SetSlice -variable sliceNumber -background #f00
   set pixrange [expr $pixmax - $pixmin]
   #puts stdout "pixrange: $pixrange"
   #scale .top.slice.level -from 1 -to $pixrange -orient horizontal \
   #  -command SetLevelBySlider -variable windo -background #f01

   
   frame .top.wl
   frame .top.wl.f1
   label .top.wl.f1.windowLabel -text "Window"
   #scale .top.wl.f1.window -from 1 -to [expr $w * 2]  -orient horizontal \
   #  -command SetWindow -variable window -background #f0f
   scale .top.wl.f1.window -from 1 -to $pixrange -orient horizontal \
     -command SetWindow -variable window -background #f0f
   frame .top.wl.f2
   label .top.wl.f2.levelLabel -text "  Level  "
   ##scale .top.wl.f2.level -from [expr $l - $w] -to [expr $l + $w] \
   ##  -orient horizontal -command SetLevel -background #f0f
   scale .top.wl.f2.level -from $pixmin -to $pixmax \
     -orient horizontal -command SetLevel -background #f0f


   # resolutions less than 1.0
   if {$w < 10} {
      set res [expr 0.05 * $w]
      .top.wl.f1.window configure -resolution $res -from $res -to [expr 2.0 * $w]
      .top.wl.f2.level configure -resolution $res \
	-from [expr 0.0 + $l - $w] -to [expr 0.0 + $l + $w] 
   }

   .top.wl.f1.window set $w
   .top.wl.f2.level set $l
   
   pack .top.slice -side top
   pack .top.slice.label .top.slice.scale -side left
   pack .top.wl.f1 .top.wl.f2 -side top

   pack .top.wl.f1.windowLabel .top.wl.f1.window -side left
   pack .top.wl.f2.levelLabel .top.wl.f2.level -side left
   pack .top.wl -side left
   ##pack forget .top.wl
   pack .top.wl -side top

   frame .top.thresh
   label .top.thresh.lab -text "INTENSITY THRESHOLDING"
   pack .top.thresh -side left
   pack .top.thresh.lab -side top
   label .top.thresh.lab2 -text "Threshold"
   if {$pixmin == $pixmax} {
      scale .top.thresh.value -from [expr $l - $w] -to [expr $l + $w] \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue \
         -resolution 0.1 -length 500
      set tvalue $l
   } else {
       set pmin [expr $pixmin - 1.0]
       set pmax [expr $pixmax + 1.0]
       scale .top.thresh.value -from $pmin -to $pmax \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue \
         -resolution 0.1 -length 500
      set tvalue [expr ($pixmax - $pixmin) / 2.0 ]
   }
   .top.thresh.value set $tvalue
   pack .top.thresh.lab2 .top.thresh.value -side left
   pack forget .top.thresh

   frame .top.iso
   button .top.iso.b1 -background #0f0 -text " Create Isosurface "  -command CreateIsosurface
   set source_file "OpenVolumeRender.tcl"
   #if {$app_type == "exe"} { 
   #   button .top.iso.volb -background #0ff -text " Volume Surface Render " -command "source $source_file"
   #} else {
   #   button .top.iso.volb -background #0ff -text " Volume Surface Render " -command "source $landmarker_home$source_file"
   #}
   #pack .top.iso.b1 .top.iso.volb
   pack .top.iso.b1
   pack forget .top.iso 
   
   frame .top.shrink
   scale .top.shrink.x -from 1 -to 10 -orient horizontal -variable XShrinkFactor -resolution 1 -length 100
   label .top.shrink.xtxt -text "Shrink Factor X"
   label .top.shrink.ytxt -text "Shrink Factor Y"
   label .top.shrink.ztxt -text "Shrink Factor Z"
   tk_optionMenu .top.shrink.interp menubuttonoptions "Interpolation Off" "Interpolation On"
   label .top.shrink.decitxt -text "Decimate"
   scale .top.shrink.y -from 1 -to 10 -orient horizontal -variable YShrinkFactor -resolution 1 -length 100
   scale .top.shrink.z -from 1 -to 10 -orient horizontal -variable ZShrinkFactor -resolution 1 -length 100 
   scale .top.shrink.deci -from 0.0 -to 1.0 -orient horizontal -variable deciFactor -resolution 0.1 -length 100 
   pack .top.shrink.decitxt .top.shrink.deci .top.shrink.interp .top.shrink.ztxt .top.shrink.z .top.shrink.ytxt .top.shrink.y .top.shrink.xtxt .top.shrink.x -side bottom -fill y
   pack forget .top.shrink -side top -fill y
   set XShrinkFactor 2
   set YShrinkFactor 2
   set ZShrinkFactor 1
   set deciFactor 0.0
   .top.shrink.x set $XShrinkFactor
   .top.shrink.y set $YShrinkFactor
   .top.shrink.z set $ZShrinkFactor
   .top.shrink.deci set $deciFactor

# annotation frames
#frame .top.f2
#frame .top.f2.f1
#frame .top.f2.f2
#frame .top.f2.f3
#frame .top.f2.f4

#label .top.f2.f1.label -relief sunken 
#label .top.f2.f2.label -relief sunken 
#label .top.f2.f3.label -relief sunken 
#label .top.f2.f4.label -relief sunken 

#pack .top.f2.f1.label -fill x 
#pack .top.f2.f2.label -fill x 
#pack .top.f2.f3.label -fill x 
#pack .top.f2.f4.label -fill x 

#pack .top.f2.f1 .top.f2.f2 .top.f2.f3 .top.f2.f4 -fill x -side left -expand t
#pack .top.f1 -fill both -expand t
#pack .top.f2 -fill x 

   BindTkImageViewer .top.f1.v1  
# tie labels to variables embedded in the Tk Widget
#.top.f2.f1.label configure -textvariable [::vtk::get_widget_variable_value  .top.f1.v1 WindowLevelString]
#.top.f2.f2.label configure -textvariable [::vtk::get_widget_variable_value  .top.f1.v1 PixelPositionString]
#.top.f2.f3.label configure -textvariable [::vtk::get_widget_variable_value  .top.f1.v1 SliceString]

#.top.f2.f1.label configure -textvariable [GetWidgetVariable .top.f1.v1 WindowLevelString]
#.top.f2.f2.label configure -textvariable [GetWidgetVariable .top.f1.v1 PixelPositionString]
#.top.f2.f3.label configure -textvariable [GetWidgetVariable .top.f1.v1 SliceString]

}


proc CloseImageViewer {} {
   # Destroy the vtkImageViewer first, then the TkImageViewerWidget
   viewer Delete
   CurrentVolume Delete
   wm withdraw .top
   destroy .top
   vtkImageViewer viewer
}

proc WindowLevelGUI {} {
  global wlGUI
  if { $wlGUI } {
    pack .top.wl -side left
  } else {
    pack forget .top.wl
  }
}

#proc OpenVolumeSlicer {} {
#  source OpenVolumeSlicer.tcl
#}

proc ThresholdImageGUI_old {} {
  # Threshold the image by use of a slider.
  # Create isosurface when wanted.
  global threshGUI
  global tvalue
  set w [viewer GetColorWindow]
  set l [viewer GetColorLevel]
  if { $threshGUI } {
    pack .top.thresh -side left
    pack .top.iso -side left
    pack .top.shrink -side top -fill y
    #viewer SetInput [thresh GetOutput]
    #set tvalue $tvalue
    ThresholdImage $tvalue
    viewer Render
  } else {
    pack forget .top.thresh
    pack forget .top.iso
    pack forget .top.shrink
    viewer SetInput [CurrentVolume GetOutput]
    viewer Render
  }
}

proc ThresholdImageGUI {} {
  global tvalue pixmax pixmin

    proc CloseIntensityThresholdingMenu {} {
      viewer SetInput [CurrentVolume GetOutput]
      viewer Render
      wm withdraw .th
      destroy .th
    }
    proc ThreshOFF {} {
      global thresh
      viewer SetInput [CurrentVolume GetOutput]
      viewer Render
    }
    proc ThreshON {} {
      global tvalue
      ThresholdImage $tvalue
      viewer Render
    }

    toplevel .th
  wm title .th "Intensity Thresholding"
  set font {Helvetica 14}

  button .th.close -font $font -text  " Close "  -width 15 -foreground red -command CloseIntensityThresholdingMenu
  pack .th.close -side bottom -pady 2m

  label .th.l -font $font -justify center -text "Intensity Threshold"
  pack .th.l -side top -pady 8

  if {$pixmin == $pixmax} {
      scale .th.value -from [expr $l - $w] -to [expr $l + $w] \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue \
         -resolution 0.1 -length 500
      set tvalue $l
   } else {
       set pmin [expr $pixmin - 1.0]
       set pmax [expr $pixmax + 1.0]
       scale .th.value -from $pmin -to $pmax \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue -resolution 0.1 -length 500 -label "Intensity Threshold"
      set tvalue [expr ($pixmax - $pixmin) / 2.0 ]
   }
   .th.value set $tvalue
   pack .th.value -pady 4

   frame .th.xx
   scale .th.xx.x -from 1 -to 10 -orient horizontal -variable XShrinkFactor -resolution 1 -length 100 
   label .th.xx.xtxt -text "Shrink Factor in X"
   pack .th.xx.x .th.xx.xtxt -side left -fill y

   frame .th.yy
   scale .th.yy.y -from 1 -to 10 -orient horizontal -variable YShrinkFactor -resolution 1 -length 100 
   label .th.yy.ytxt -text "Shrink Factor in Y"
   pack .th.yy.y .th.yy.ytxt -side left -fill y

   frame .th.zz
   scale .th.zz.z -from 1 -to 10 -orient horizontal -variable ZShrinkFactor -resolution 1 -length 100 
   label .th.zz.ztxt -text "Shrink Factor in Z"
   pack .th.zz.z .th.zz.ztxt -side left -fill y

   frame .th.dd
   scale .th.dd.deci -from 0.0 -to 1.0 -orient horizontal -variable deciFactor -resolution 0.1 -length 100 
   label .th.dd.decitxt -text "Decimate"
   pack .th.dd.deci .th.dd.decitxt -side left -fill y -pady 10

   frame .th.onoff
   button .th.onoff.on -text " Threshold ON "  -foreground green -width 15 -justify center -command ThreshON
   button .th.onoff.off -text " Threshold OFF "  -foreground blue -width 15 -justify center -command ThreshOFF
   pack .th.onoff.on .th.onoff.off -side left -expand 1 -pady 10

   pack .th.xx .th.yy .th.zz .th.dd .th.onoff -fill y

   button .th.b1 -font $font -background #0f0 -text " Create Isosurface "  -foreground blue -width 15 -justify center -command CreateIsosurface
   pack .th.b1 -side left -expand 1
      
   set XShrinkFactor 2
   set YShrinkFactor 2
   set ZShrinkFactor 1
   set deciFactor 0.0
   .th.xx.x set $XShrinkFactor
   .th.yy.y set $YShrinkFactor
   .th.zz.z set $ZShrinkFactor
   .th.dd.deci set $deciFactor





}


proc ThresholdImage {tvalue} {
  global thresh uvalue
  global oactorNum
  vtkImageThreshold thresh
  thresh SetInput [CurrentVolume GetOutput]
  set w [viewer GetColorWindow]
  set l [viewer GetColorLevel]
  set uvalue [expr $l + $w]
  thresh ThresholdByUpper $tvalue 
  thresh SetInValue $uvalue
  thresh SetOutValue 0
  viewer SetInput [thresh GetOutput]
  viewer Render
  thresh Delete
}


proc SetLevelBySlider {windo} {
  global viewer 
  viewer SetColorLevel $windo
  viewer Render
}
   
proc SetSlice { slice } {
   global sliceNumber viewer

   viewer SetZSlice $slice
   viewer Render
}

proc SetWindow window {
   global viewer video
   if {$video} {
      viewer SetColorWindow [expr -$window]
   } else {
      viewer SetColorWindow $window
   }
   viewer Render
}

proc SetLevel level {
   global viewer
   viewer SetColorLevel $level
   viewer Render
}

proc SetInverseVideo {} {
   global viewer video window
   if {$video} {
      viewer SetColorWindow [expr -$window]
   } else {
      viewer SetColorWindow $window
   }
   viewer Render
}

InitializeWindowLevelInterface $infilename $xs $ys

} else {
  viewer Render
}

proc DisplayImageDimensions {} {
  global xpix ypix zpix xs ys zs
  set mess "Pixel size: $xpix $ypix $zpix , Npix: $xs $ys $zs"
  notify_message $mess
}
proc DisplayImageIntensities {} {
  global pixmin pixmax
  set mess "Intensity range min max: $pixmin , $pixmax"
  notify_message $mess
}



