# Set up landmarker image viewer.
# Places in the tcl top window.  Looks for object named viewer.

if {[info commands rtExMath] != "rtExMath"} {

source ViewerAppTkImageViewerInteractor.tcl

# Take window level parameters from viewer
proc InitializeWindowLevelInterface {infilename xs ys} {
   global viewer sliceNumber
   global pixmin pixmax

   # Get parameters from viewer
   set w [viewer GetColorWindow]
   set l [viewer GetColorLevel]
   set sliceNumber [viewer GetZSlice]
   set zMin [viewer GetWholeZMin]
   set zMax [viewer GetWholeZMax]

   #notify $w w
   #notify $l l



# Create the image viewer user interface.

destroy .top
toplevel .top -visual best
set filename "Image Viewer: [file tail $infilename]"
wm title .top $filename  

menu .top.menu -type menubar

menu .top.menu.file -tearoff 1
menu .top.menu.edit -tearoff 1
menu .top.menu.options -tearoff 1
menu .top.menu.help -tearoff 1

.top.menu add cascade -menu .top.menu.file -label "File"
.top.menu add cascade -menu .top.menu.edit -label "Edit"
.top.menu add cascade -menu .top.menu.options -label "Options"
.top.menu add cascade -menu .top.menu.help -label "Help"

.top.menu.file add command -label Exit -command CloseImageViewer -foreground red
.top.menu.options add check -label "Threshold/Isosurface" -command ThresholdImageGUI -variable threshGUI 
.top.menu.options add check -label "Reverse Colors" -command SetInverseVideo -variable video
.top.menu.options add check -label "Open Volume Slicer" -command "source OpenVolumeSlicer.tcl" -variable VolumeSlicer
#.top.menu.options add check -label "Window/Level Adjust" -command WindowLevelGUI -variable wlGUI

.top configure -menu .top.menu

# The rendering widget
frame .top.f1
vtkTkImageViewerWidget .top.f1.v1 -width $xs -height $ys -iv viewer
pack .top.f1.v1 -padx 3 -pady 3 -side left -fill both -expand t
pack .top.f1 -fill both -expand t

   frame .top.slice
   label .top.slice.label -text " Slice # "
   scale .top.slice.scale -from $zMin -to $zMax -orient horizontal \
     -command SetSlice -variable sliceNumber -background #f00
   
   frame .top.wl
   frame .top.wl.f1
   label .top.wl.f1.windowLabel -text "Window"
   scale .top.wl.f1.window -from 1 -to [expr $w * 2]  -orient horizontal \
     -command SetWindow -variable window -background #f0f
   frame .top.wl.f2
   label .top.wl.f2.levelLabel -text "  Level  "
   scale .top.wl.f2.level -from [expr $l - $w] -to [expr $l + $w] \
     -orient horizontal -command SetLevel -background #f0f

   # resolutions less than 1.0
   if {$w < 10} {
      set res [expr 0.05 * $w]
      .top.wl.f1.window configure -resolution $res -from $res -to [expr 2.0 * $w]
      .top.wl.f2.level configure -resolution $res \
	-from [expr 0.0 + $l - $w] -to [expr 0.0 + $l + $w] 
   }

   .top.wl.f1.window set $w
   .top.wl.f2.level set $l
   
   pack .top.slice -side top
   pack .top.slice.label .top.slice.scale -side left
   pack .top.wl.f1 .top.wl.f2 -side top

   pack .top.wl.f1.windowLabel .top.wl.f1.window -side left
   pack .top.wl.f2.levelLabel .top.wl.f2.level -side left
   pack .top.wl -side left
   pack forget .top.wl

   frame .top.thresh
   label .top.thresh.lab -text "THRESHOLDING"
   pack .top.thresh -side left
   pack .top.thresh.lab -side top
   label .top.thresh.lab2 -text "Threshold"
   if {$pixmin == $pixmax} {
      scale .top.thresh.value -from [expr $l - $w] -to [expr $l + $w] \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue \
         -resolution 0.1 
      set tvalue $l
   } else {
       scale .top.thresh.value -from $pixmin -to $pixmax \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue \
         -resolution 0.1 
      set tvalue [expr ($pixmax - $pixmin) / 2.0 ]
   }
   .top.thresh.value set $tvalue
   pack .top.thresh.lab2 .top.thresh.value -side left
   pack forget .top.thresh

   frame .top.iso
   button .top.iso.b1 -text  " Create Isosurface "  -command CreateIsosurface
   pack .top.iso.b1
   pack forget .top.iso 

# annotation frames
frame .top.f2
frame .top.f2.f1
frame .top.f2.f2
frame .top.f2.f3
frame .top.f2.f4

label .top.f2.f1.label -relief sunken 
label .top.f2.f2.label -relief sunken 
label .top.f2.f3.label -relief sunken 
label .top.f2.f4.label -relief sunken 

pack .top.f2.f1.label -fill x 
pack .top.f2.f2.label -fill x 
pack .top.f2.f3.label -fill x 
pack .top.f2.f4.label -fill x 

pack .top.f2.f1 .top.f2.f2 .top.f2.f3 .top.f2.f4 -fill x -side left -expand t
pack .top.f1 -fill both -expand t
pack .top.f2 -fill x 

   BindTkImageViewer .top.f1.v1  
# tie labels to variables embedded in the Tk Widget
.top.f2.f1.label configure -textvariable [GetWidgetVariable .top.f1.v1 WindowLevelString]
.top.f2.f2.label configure -textvariable [GetWidgetVariable .top.f1.v1 PixelPositionString]
.top.f2.f3.label configure -textvariable [GetWidgetVariable .top.f1.v1 SliceString]

}


proc CloseImageViewer {} {
   wm withdraw .top
   destroy .top
}

proc WindowLevelGUI {} {
  global wlGUI
  if { $wlGUI } {
    pack .top.wl -side left
  } else {
    pack forget .top.wl
  }
}

#proc OpenVolumeSlicer {} {
#  source OpenVolumeSlicer.tcl
#}

proc ThresholdImageGUI {} {
  # Threshold the image by use of a slider.
  # Create isosurface when wanted.
  global reslice1 tvalue
  set w [viewer GetColorWindow]
  set l [viewer GetColorLevel]
  global threshGUI
  if { $threshGUI } {
    pack .top.thresh -side left
    pack .top.iso -side left
    #viewer SetInput [thresh GetOutput]
    #set tvalue $tvalue
    notify $tvalue tvalue
    ThresholdImage $tvalue
    viewer Render
  } else {
    pack forget .top.thresh
    pack forget .top.iso
    viewer SetInput [reslice1 GetOutput]
    viewer Render
  }
  
  
}

proc ThresholdImage {tvalue} {
  global thresh uvalue
  global oactorNum
  vtkImageThreshold thresh
  thresh SetInput [reslice1 GetOutput]
  set w [viewer GetColorWindow]
  set l [viewer GetColorLevel]
  set uvalue [expr $l + $w]
  thresh ThresholdByUpper $tvalue 
  thresh SetInValue $uvalue
  thresh SetOutValue 0
  viewer SetInput [thresh GetOutput]
  viewer Render
  thresh Delete
}

proc CreateIsosurface {} {
  global renWin ren1
  global tvalue
  global lmDiameter
  global oactorNum
  #vtkMarchingContourFilter iso
  #iso SetUseScalarTree 1
  #iso SetComputeScalars 0
  #iso SetComputeNormals 1
  #iso SetInput [reslice1 GetOutput]
  #iso SetValue 0 $tvalue

  [reslice1 GetOutput] SetOrigin 0.0 0.0 0.0
  vtkImageMarchingCubes iso
    iso SetInput [reslice1 GetOutput]
    iso SetValue 0 1150
    iso SetInputMemoryLimit 1000
  

  vtkPolyDataMapper isoMapper
    isoMapper SetInput [iso GetOutput]
    isoMapper ScalarVisibilityOff

  oactorPoly$oactorNum Delete
  vtkTransformPolyDataFilter oactorPoly$oactorNum
  oactorPoly$oactorNum SetTransform identityTransform
  oactorPoly$oactorNum SetInput [iso GetOutput]

  #vtkPolyDataNormals isonormals
  #  isonormals SetInput [iso GetOutput]
  #  set norms [isonormals GetOutput]
 
  #vtkPolyDataWriter writer
  #  writer SetFileName t.vtk
  #  #writer SetInput [oactorPoly$oactorNum GetOutput]
  #  writer SetInput [iso GetOutput]
  #  writer Write
    
  vtkActor isoActor
  isoActor SetMapper isoMapper
  [isoActor GetProperty] SetColor 1 1 1
  [isoActor GetProperty] SetAmbient 0.26
  [isoActor GetProperty] SetDiffuse 0.42
  [isoActor GetProperty] SetSpecular 0.26
  [isoActor GetProperty] SetSpecularPower 79.0
  set Bounds [isoActor GetBounds]
    set Xmin [lindex $Bounds 0] 
    set Xmax [lindex $Bounds 1] 
    set Ymin [lindex $Bounds 2] 
    set Ymax [lindex $Bounds 3] 
    set Zmin [lindex $Bounds 4] 
    set Zmax [lindex $Bounds 5] 
    set fact 0.015
    set lmDiameter [expr $fact * (($Xmax - $Xmin) + ($Ymax - $Ymin) + ($Zmax - $Zmin))/3.0]
    lmsphere0 SetRadius $lmDiameter

  #vtkOutlineFilter outline
  #    outline SetInput [reslice1 GetOutput]
  #vtkPolyDataMapper outlineMapper
  #    outlineMapper SetInput [outline GetOutput]
  #vtkActor outlineActor
  #    outlineActor SetMapper outlineMapper
  #    outlineActor VisibilityOff

  # Add the actors to the renderer, set the background and size
  #
  #ren1 AddActor outlineActor
  ren1 AddActor isoActor
  ren1 ResetCamera
  renWin Render
}

   
proc SetSlice { slice } {
   global sliceNumber viewer

   viewer SetZSlice $slice
   viewer Render
}

proc SetWindow window {
   global viewer video
   if {$video} {
      viewer SetColorWindow [expr -$window]
   } else {
      viewer SetColorWindow $window
   }
   viewer Render
}

proc SetLevel level {
   global viewer
   viewer SetColorLevel $level
   viewer Render
}

proc SetInverseVideo {} {
   global viewer video window
   if {$video} {
      viewer SetColorWindow [expr -$window]
   } else {
      viewer SetColorWindow $window
   }
   viewer Render
}

InitializeWindowLevelInterface $infilename $xs $ys

} else {
  viewer Render
}

