# Set up landmarker image viewer.
# Places in the tcl top window.  Looks for object named viewer.

if {[info commands rtExMath] != "rtExMath"} {

source ViewerAppTkImageViewerInteractor.tcl

# Take window level parameters from viewer
proc InitializeWindowLevelInterface {infilename xs ys} {
   global viewer sliceNumber
   global pixmin pixmax

   # Get parameters from viewer
   set w [viewer GetColorWindow]
   set l [viewer GetColorLevel]
   set sliceNumber [viewer GetZSlice]
   set zMin [viewer GetWholeZMin]
   set zMax [viewer GetWholeZMax]

   
# Create the image viewer user interface.

##wm withdraw .
##set top [toplevel .top] 
##wm protocol .top WM_DELETE_WINDOW ::vtk::cb_exit
##wm title .top "Mandelbrot Viewer"

destroy .top
toplevel .top -visual best

set filename "Image Viewer: [file tail $infilename]"
wm title .top $filename  

menu .top.menu -type menubar

menu .top.menu.file -tearoff 1
menu .top.menu.edit -tearoff 1
menu .top.menu.options -tearoff 1
menu .top.menu.help -tearoff 1

.top.menu add cascade -menu .top.menu.file -label "File"
.top.menu add cascade -menu .top.menu.edit -label "Edit"
.top.menu add cascade -menu .top.menu.options -label "Options"
.top.menu add cascade -menu .top.menu.help -label "Help"

.top.menu.file add command -label Exit -command CloseImageViewer -foreground red
##.top.menu.file add command -label Exit -command ::vtk::cb_exit -foreground red
.top.menu.options add check -label "Threshold/Isosurface" -command ThresholdImageGUI -variable threshGUI 
.top.menu.options add check -label "Reverse Colors" -command SetInverseVideo -variable video
.top.menu.options add check -label "Volume Slicer" -command "source OpenVolumeSlicer.tcl" -variable VolumeSlicer
.top.menu.options add check -label "Volume Render" -command "source OpenVolumeRender.tcl" -variable VolumeRenderWindow

##.top.menu.options add check -label "Window/Level Adjust" -command WindowLevelGUI -variable wlGUI

.top configure -menu .top.menu

# The rendering widget
frame .top.f1
vtkTkImageViewerWidget .top.f1.v1 -width $xs -height $ys -iv viewer
pack .top.f1.v1 -padx 3 -pady 3 -side left -fill both -expand t
pack .top.f1 -fill both -expand t

   frame .top.slice
   label .top.slice.label -text " Slice # "
   scale .top.slice.scale -from $zMin -to $zMax -orient horizontal \
     -command SetSlice -variable sliceNumber -background #f00
   
   frame .top.wl
   frame .top.wl.f1
   label .top.wl.f1.windowLabel -text "Window"
   scale .top.wl.f1.window -from 1 -to [expr $w * 2]  -orient horizontal \
     -command SetWindow -variable window -background #f0f
   frame .top.wl.f2
   label .top.wl.f2.levelLabel -text "  Level  "
   scale .top.wl.f2.level -from [expr $l - $w] -to [expr $l + $w] \
     -orient horizontal -command SetLevel -background #f0f

   # resolutions less than 1.0
   if {$w < 10} {
      set res [expr 0.05 * $w]
      .top.wl.f1.window configure -resolution $res -from $res -to [expr 2.0 * $w]
      .top.wl.f2.level configure -resolution $res \
	-from [expr 0.0 + $l - $w] -to [expr 0.0 + $l + $w] 
   }

   .top.wl.f1.window set $w
   .top.wl.f2.level set $l
   
   pack .top.slice -side top
   pack .top.slice.label .top.slice.scale -side left
   pack .top.wl.f1 .top.wl.f2 -side top

   pack .top.wl.f1.windowLabel .top.wl.f1.window -side left
   pack .top.wl.f2.levelLabel .top.wl.f2.level -side left
   pack .top.wl -side left
   pack forget .top.wl

   frame .top.thresh
   label .top.thresh.lab -text "INTENSITY THRESHOLDING"
   pack .top.thresh -side left
   pack .top.thresh.lab -side top
   label .top.thresh.lab2 -text "Threshold"
   if {$pixmin == $pixmax} {
      scale .top.thresh.value -from [expr $l - $w] -to [expr $l + $w] \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue \
         -resolution 0.1 -length 500
      set tvalue $l
   } else {
       scale .top.thresh.value -from $pixmin -to $pixmax \
         -orient horizontal -command ThresholdImage -background #0f0  -variable tvalue \
         -resolution 0.1 -length 500
      set tvalue [expr ($pixmax - $pixmin) / 2.0 ]
   }
   .top.thresh.value set $tvalue
   pack .top.thresh.lab2 .top.thresh.value -side left
   pack forget .top.thresh

   frame .top.iso
   button .top.iso.b1 -background #0f0 -text " Create Isosurface "  -command CreateIsosurface
   pack .top.iso.b1
   pack forget .top.iso 
   
   frame .top.shrink
   scale .top.shrink.x -from 1 -to 10 -orient horizontal -variable XShrinkFactor -resolution 1 -length 100
   label .top.shrink.xtxt -text "Shrink Factor X"
   label .top.shrink.ytxt -text "Shrink Factor Y"
   label .top.shrink.ztxt -text "Shrink Factor Z"
   tk_optionMenu .top.shrink.interp menubuttonoptions "Interpolation Off" "Interpolation On"
   label .top.shrink.decitxt -text "Decimate"
   scale .top.shrink.y -from 1 -to 10 -orient horizontal -variable YShrinkFactor -resolution 1 -length 100
   scale .top.shrink.z -from 1 -to 10 -orient horizontal -variable ZShrinkFactor -resolution 1 -length 100 
   scale .top.shrink.deci -from 0.0 -to 1.0 -orient horizontal -variable deciFactor -resolution 0.1 -length 100 
   pack .top.shrink.decitxt .top.shrink.deci .top.shrink.interp .top.shrink.ztxt .top.shrink.z .top.shrink.ytxt .top.shrink.y .top.shrink.xtxt .top.shrink.x -side bottom -fill y
   pack forget .top.shrink -side top -fill y
   set XShrinkFactor 2
   set YShrinkFactor 2
   set ZShrinkFactor 1
   set deciFactor 0.0
   .top.shrink.x set $XShrinkFactor
   .top.shrink.y set $YShrinkFactor
   .top.shrink.z set $ZShrinkFactor
   .top.shrink.deci set $deciFactor

# annotation frames
frame .top.f2
frame .top.f2.f1
frame .top.f2.f2
frame .top.f2.f3
frame .top.f2.f4

label .top.f2.f1.label -relief sunken 
label .top.f2.f2.label -relief sunken 
label .top.f2.f3.label -relief sunken 
label .top.f2.f4.label -relief sunken 

pack .top.f2.f1.label -fill x 
pack .top.f2.f2.label -fill x 
pack .top.f2.f3.label -fill x 
pack .top.f2.f4.label -fill x 

pack .top.f2.f1 .top.f2.f2 .top.f2.f3 .top.f2.f4 -fill x -side left -expand t
pack .top.f1 -fill both -expand t
pack .top.f2 -fill x 

   BindTkImageViewer .top.f1.v1  
# tie labels to variables embedded in the Tk Widget
.top.f2.f1.label configure -textvariable [::vtk::get_widget_variable_value  .top.f1.v1 WindowLevelString]
.top.f2.f2.label configure -textvariable [::vtk::get_widget_variable_value  .top.f1.v1 PixelPositionString]
.top.f2.f3.label configure -textvariable [::vtk::get_widget_variable_value  .top.f1.v1 SliceString]

##.top.f2.f1.label configure -textvariable [GetWidgetVariable .top.f1.v1 WindowLevelString]
##.top.f2.f2.label configure -textvariable [GetWidgetVariable .top.f1.v1 PixelPositionString]
##.top.f2.f3.label configure -textvariable [GetWidgetVariable .top.f1.v1 SliceString]

}


proc CloseImageViewer {} {
   # Destroy the vtkImageViewer first, then the TkImageViewerWidget
   viewer Delete
   reslice2 Delete
   wm withdraw .top
   destroy .top
   vtkImageViewer viewer
}

proc WindowLevelGUI {} {
  global wlGUI
  if { $wlGUI } {
    pack .top.wl -side left
  } else {
    pack forget .top.wl
  }
}

#proc OpenVolumeSlicer {} {
#  source OpenVolumeSlicer.tcl
#}

proc ThresholdImageGUI {} {
  # Threshold the image by use of a slider.
  # Create isosurface when wanted.
  global reslice2 tvalue
  set w [viewer GetColorWindow]
  set l [viewer GetColorLevel]
  global threshGUI
  if { $threshGUI } {
    pack .top.thresh -side left
    pack .top.iso -side left
    pack .top.shrink -side top -fill y
    #viewer SetInput [thresh GetOutput]
    #set tvalue $tvalue
    ThresholdImage $tvalue
    viewer Render
  } else {
    pack forget .top.thresh
    pack forget .top.iso
    pack forget .top.shrink
    viewer SetInput [reslice2 GetOutput]
    viewer Render
  }
  
  
}

proc ThresholdImage {tvalue} {
  global thresh uvalue
  global oactorNum
  vtkImageThreshold thresh
  thresh SetInput [reslice2 GetOutput]
  set w [viewer GetColorWindow]
  set l [viewer GetColorLevel]
  set uvalue [expr $l + $w]
  thresh ThresholdByUpper $tvalue 
  thresh SetInValue $uvalue
  thresh SetOutValue 0
  viewer SetInput [thresh GetOutput]
  viewer Render
  thresh Delete
}

proc CreateIsosurface {} {
  global renWin ren1
  global tvalue
  global lmDiameter
  global oactorNum
  global XShrinkFactor YShrinkFactor ZShrinkFactor deciFactor
  #vtkMarchingContourFilter iso
  #iso SetUseScalarTree 1
  #iso SetComputeScalars 0
  #iso SetComputeNormals 1
  #iso SetInput [reslice2 GetOutput]
  #iso SetValue 0 $tvalue

  # Shrink image before Marching Cubes.
  set sum [expr $XShrinkFactor + $YShrinkFactor + $ZShrinkFactor ]
  if {$sum != 3} {

  vtkImageShrink3D shrink 
    shrink SetShrinkFactors $XShrinkFactor $YShrinkFactor $ZShrinkFactor 
    shrink SetInput [reslice2 GetOutput]
    shrink AveragingOff

  [shrink GetOutput] SetOrigin 0.0 0.0 0.0
  vtkImageMarchingCubes iso
    iso SetInput [shrink GetOutput]
    iso SetValue 0 $tvalue
    iso SetInputMemoryLimit 1000000000
  }

    # Move the isosurface data into PolyData
    iso Update
      PolyData CopyStructure [iso GetOutput]
      [PolyData GetPointData] PassData [[iso GetOutput] GetPointData]
      PolyData Modified

  # Decimate also, if wanted.
  if {$deciFactor != 0} {
  vtkDecimatePro deci
    deci SetInput PolyData   
    deci SetTargetReduction $deciFactor
    deci PreserveTopologyOn
    deci Update
      PolyData CopyStructure [deci GetOutput]
      [PolyData GetPointData] PassData [[deci GetOutput] GetPointData]
      PolyData Modified
      deci Delete
   }
iso Delete
rmapper$oactorNum Delete
oactor$oactorNum Delete
vtkPolyDataMapper rmapper$oactorNum
  rmapper$oactorNum SetInput PolyData 
vtkActor oactor$oactorNum 
  oactor$oactorNum SetMapper rmapper$oactorNum

.menubar.mSurface entryconfigure 1 -state normal

oactor$oactorNum SetMapper rmapper$oactorNum
    ##rmapper$oactorNum ScalarVisibilityOff
    #[oactor$oactorNum GetProperty] SetColor 1 1 1
    #[oactor$oactorNum GetProperty] SetAmbient 0.26
    #[oactor$oactorNum GetProperty] SetDiffuse 0.42
    #[oactor$oactorNum GetProperty] SetSpecular 0.26
    #[oactor$oactorNum GetProperty] SetSpecularPower 79.0
    [oactor$oactorNum GetProperty] SetColor 1 1 1
    [oactor$oactorNum GetProperty] SetAmbient 0.20
    [oactor$oactorNum GetProperty] SetDiffuse 0.40
    [oactor$oactorNum GetProperty] SetSpecular 0.20
    [oactor$oactorNum GetProperty] SetSpecularPower 50.0
    [oactor$oactorNum GetProperty] SetOpacity 1.0
    set Bounds [oactor$oactorNum GetBounds]
    set Xmin [lindex $Bounds 0] 
    set Xmax [lindex $Bounds 1] 
    set Ymin [lindex $Bounds 2] 
    set Ymax [lindex $Bounds 3] 
    set Zmin [lindex $Bounds 4] 
    set Zmax [lindex $Bounds 5] 
    set fact 0.015
    set lmDiameter [expr $fact * (($Xmax - $Xmin) + ($Ymax - $Ymin) + ($Zmax - $Zmin))/3.0]



ren1 AddActor oactor$oactorNum
#DefaultProperties

   ren1 ResetCamera
  renWin Render
}

   
proc SetSlice { slice } {
   global sliceNumber viewer

   viewer SetZSlice $slice
   viewer Render
}

proc SetWindow window {
   global viewer video
   if {$video} {
      viewer SetColorWindow [expr -$window]
   } else {
      viewer SetColorWindow $window
   }
   viewer Render
}

proc SetLevel level {
   global viewer
   viewer SetColorLevel $level
   viewer Render
}

proc SetInverseVideo {} {
   global viewer video window
   if {$video} {
      viewer SetColorWindow [expr -$window]
   } else {
      viewer SetColorWindow $window
   }
   viewer Render
}

InitializeWindowLevelInterface $infilename $xs $ys

} else {
  viewer Render
}

