package require vtk
package require vtkinteraction

# Uses the Hume tcl math library.
# See http://www.hume.com/la/

# Vectors should be in the Hume math library format.
# E.g. the vector A with elements 15,16,17 becomes
# set A [list 2 1 0 15 16 17]  (2 is the number of extra elements for size information, 1 is number of rows, 0 is number of columns)
# 

package require La
##namespace import La::*

proc crossproduct {vec1 vec2} {
  set i 0
  set index [expr {3 + $i}]
  set u0 [lindex $vec1 $index]
  set i 1
  set index [expr {3 + $i}]
  set u1 [lindex $vec1 $index]
  set i 2
  set index [expr {3 + $i}]
  set u2 [lindex $vec1 $index]
  set i 0
  set index [expr {3 + $i}]
  set v0 [lindex $vec2 $index]
  set i 1
  set index [expr {3 + $i}]
  set v1 [lindex $vec2 $index]
  set i 2
  set index [expr {3 + $i}]
  set v2 [lindex $vec2 $index]
  set cp0 [expr {$u1 * $v2 - $u2 * $v1}]
  set cp1 [expr {$u2 * $v0 - $u0 * $v2}]
  set cp2 [expr {$u0 * $v1 - $u1 * $v0}]
  set cp [list 2 1 0 $cp0 $cp1 $cp2]
  return $cp
}

proc distance { p1 p2 } {
  # Computes distance between two points.
  set i 0
  set index [expr {3 + $i}]
  set p1x [lindex $p1 $index]
  set i 1
  set index [expr {3 + $i}]
  set p1y [lindex $p1 $index]
  set i 2
  set index [expr {3 + $i}]
  set p1z [lindex $p1 $index]
  set i 0
  set index [expr {3 + $i}]
  set p2x [lindex $p2 $index]
  set i 1
  set index [expr {3 + $i}]
  set p2y [lindex $p2 $index]
  set i 2
  set index [expr {3 + $i}]
  set p2z [lindex $p2 $index]
  set dx2 [expr {($p2x - $p1x) * ($p2x - $p1x)}]
  set dy2 [expr {($p2y - $p1y) * ($p2y - $p1y)}]
  set dz2 [expr {($p2z - $p1z) * ($p2z - $p1z)}]
  set d [expr {sqrt($dx2 + $dy2 + $dz2)}]
  return $d


}

proc vectorlength {vector} {
  set i 0
  set index [expr {3 + $i}]
  set x [lindex $vector $index]
  set i 1
  set index [expr {3 + $i}]
  set y [lindex $vector $index]
  set i 2
  set index [expr {3 + $i}]
  set z [lindex $vector $index]

  set lx2 [expr $x * $x]
  set ly2 [expr $y * $y]
  set lz2 [expr $z * $z]

  set length [expr sqrt ( $lx2 + $ly2 + $lz2   ) ]

  return $length
}

proc pnt_line {p0 l0 l1} {
  # Routine returns a list containing {distance plx ply plz},
  # i.e. the perpendicular distance from point p0 to line defined by l0 and l1,
  # and the point pl on the line being closest to p0.
  
  set lv [msub $l1 $l0]
  ##puts stdout "lv $lv"
  set l [vectorlength $lv]
  ##puts stdout "l $l"
  set lm [expr 1.0 / $l]
  set lv [mscale $lv $lm]
  ##puts stdout "lv $lv"

  set up1 [mprod $lv $p0]
  set up1 [msum $up1]
  set up1 [expr $up1 * -1.0]
  ##puts stdout "up1 $up1"

  set up2 [mprod $l0 $lv]
  set up2 [msum $up2]
  set up [expr $up1 + $up2]
  ##puts stdout "up $up"
  set lo [mprod $lv $lv]
  set lo [msum $lo]
  set t [expr $up / $lo]
  set t [expr -1.0 * $t]
  ##puts stdout "t $t"

  # Compute point on line, pl
  set tlv [mscale $lv $t]
  set pl [madd $tlv $l0]
  puts stdout "pl $pl"
 
  # Compute distance, d
  set di [msub $p0 $pl]
  set di2 [mprod $di $di]
  set dit [msum $di2]
  set d [expr sqrt($dit)]

  # We return a list containing both d and pl.
  set plx [lindex $pl 3]
  set ply [lindex $pl 4]
  set plz [lindex $pl 5]
    
  return [list $d $plx $ply $plz]
}






#wm withdraw .
